<?php
/**
 * 销售物料数据验证Lib类
 * 
 * @access public
 * @author wangbiao<wangbiao@shopex.cn>
 * @version $Id: check.php 2016-08-03 15:00
 */
class material_sales_check
{
    /**
     * 数据检验有效性
     * 
     * @param  Array   $params
     * @param  String  $err_msg
     * @return Boolean
     */
    public function checkParams(&$params, &$err_msg)
    {
        $salesMaterialObj = app::get('material')->model('sales_material');
        
        //新增标记
        $is_new_add    = $params['edit'] ? false : true;
        unset($params['edit']);
        
        if(empty($params['sales_material_name'])){
            $err_msg ="销售物料名称不能为空";
            return false;
        }
        
        if(empty($params['sales_material_bn'])){
            $err_msg ="销售物料编码不能为空";
            return false;
        }
        
        //检查有效性
        if($is_new_add)
        {
            //判断物料编码只能是由数字英文下划线组成
            $reg_bn_code = "/^[0-9a-zA-Z\_\-]*$/";
            if(!preg_match($reg_bn_code,$params["sales_material_bn"])){
                $err_msg = "物料编码只能是数字英文下划线组成";
                return false;
            }
            
            $salesMaterialInfo = $salesMaterialObj->getList('sales_material_bn',array('sales_material_bn'=>$params['sales_material_bn']));
            if($salesMaterialInfo){
                $err_msg ="当前新增的物料编码已被使用，不能重复";
                return false;
            }
            
            $params['sales_material_bn_crc32'] = sprintf('%u',crc32($params['sales_material_bn']));
        }
        else 
        {
            if(empty($params['sm_id'])){
                $err_msg ="销售物料sm_id不能为空";
                return false;
            }
            
            $salesMaterialExistInfo = $salesMaterialObj->getList('sm_id',array('sales_material_bn'=>$params['sales_material_bn']));
            if($salesMaterialExistInfo && $salesMaterialExistInfo[0]['sm_id'] != $params['sm_id']){
                $err_msg ="当前编辑的物料编码已被使用，不能重复";
                return false;
            }
            
            $salesMaterialInfo = $salesMaterialObj->dump($params['sm_id']);
            if(!$salesMaterialInfo){
                $err_msg ="当前物料不存在";
                return false;
            }
        }
        
        if($params['sales_material_type'] == 2){
            if(!isset($params['at'])){
                $err_msg ="促销物料请至少设置一个物料明细内容";
                return false;
            }

            foreach ($params['at'] as $val){
                if (count($params['at']) == 1){
                    if ($val <2){
                        $err_msg ="只有一种物料时，数量必须大于1";
                        return false;
                    }
                }else {
                    if ($val < 1){
                        $err_msg ="数量必须大于0";
                        return false;
                    }
                }
            }

            foreach ($params['pr'] as $val){
                $tmp_rate +=$val;
            }

            if($tmp_rate > 100){
                $err_msg ="分摊销售价合计百分比:".$tmp_rate.",已超100%";
                return false;
            }elseif($tmp_rate < 100){
                $err_msg ="分摊销售价合计百分比:".$tmp_rate.",不足100%";
                return false;
            }
        }
        
        //福袋
        if($params['sales_material_type'] == 4){
            
            //编辑操作 如果福袋产生过交易订单则不能编辑（也因为由于显示html不同 无post值） 无需走下面的福袋判断
            if(!$is_new_add && $params["can_edit_luckybag_rules"] == "no"){
                return true;
            }
            
            if(empty($params["lbr"])){
                $err_msg ="福袋物料请至少设置一组福袋规则";
                return false;
            }
            $reg_number = "/^[1-9][0-9]*$/";
            $luckybag_total_price = 0;
            $luckybag_error_message = array();
            $arr_lbr_name = array();
            $bm_ids_record = array();
            foreach($params["lbr"] as $var_lbr){
                if(!$var_lbr["name"] && !isset($luckybag_error_message["empty_check"])){
                    $luckybag_error_message["empty_check"] = "组合名称不能为空";
                }
                if(in_array($var_lbr["name"],$arr_lbr_name) && $var_lbr["name"] && !isset($luckybag_error_message["same_check"])){
                    $luckybag_error_message["same_check"] = "组合名称不能相同";
                }else{
                    $arr_lbr_name[] = $var_lbr["name"];
                }
                if(empty($var_lbr["bm_ids"]) && !isset($luckybag_error_message["empty_bm_ids"])){
                    $luckybag_error_message["empty_bm_ids"] = "组合的基础物料不能为空";
                }else{
                    foreach($var_lbr["bm_ids"] as $var_bi){
                        if(in_array($var_bi,$bm_ids_record) && !isset($luckybag_error_message["duplicate_bm_id"])){
                            $luckybag_error_message["duplicate_bm_id"] = "所有组合中基础物料不得重复";
                        }else{
                            $bm_ids_record[] = $var_bi;
                        }
                    }
                }
                if(!preg_match($reg_number,$var_lbr["sku_num"])){
                    $luckybag_error_message["error_sku_num"] = "sku数必须是大于0的整数";
                }
                if(!preg_match($reg_number,$var_lbr["send_num"])){
                    $luckybag_error_message["error_send_num"] = "送件数必须是大于0的整数";
                }
                if($var_lbr["sku_num"] > count($var_lbr["bm_ids"])){
                    $luckybag_error_message["error_sku_num"] = "sku数不得大于当前基础物料sku数量";
                }
                if(!is_numeric($var_lbr["single_price"]) || $var_lbr["single_price"] < 0){
                    $luckybag_error_message["error_single_price"] = "单品售价必须是大于等于0的数字";
                }
                $luckybag_total_price = $luckybag_total_price + ($var_lbr["sku_num"]*$var_lbr["send_num"]*$var_lbr["single_price"]);
            }
            if($params["retail_price"] != $luckybag_total_price){
                $luckybag_error_message["price_check"] = "售价和组合总和售价必须相等";
            }
            if(!empty($luckybag_error_message)){
                $err_msg = implode("<br />", $luckybag_error_message);
                return false;
            }
            
            $filter_luckybag_exists = array("lbr_name"=>$arr_lbr_name);
            if($params['sm_id'] && !$is_new_add){ //编辑
                $filter_luckybag_exists["sm_id|noequal"] = $params['sm_id'];
            }
            
            $mdl_material_luckybag_rules = app::get('material')->model('luckybag_rules');
            $rs_lbr = $mdl_material_luckybag_rules->getList("lbr_name",$filter_luckybag_exists);
            if(!empty($rs_lbr)){
                $exist_lbr_names = array();
                foreach($rs_lbr as $var_r_l){
                    $exist_lbr_names[] = $var_r_l["lbr_name"];
                }
                $err_msg = "组合：".implode(",",$exist_lbr_names)."已经存在";
                return false;
            }
        }
        
        return true;
    }

    /**
     * 检查基础物料个别参数是否可编辑
     * 
     * @param Int $bm_id
     * @return Array
     */
    function checkEditReadOnly($shop_id, $sm_id, $is_bind){
        $readonly = array('type' => false,'shop'=>false,'bind_item'=>false);

        //如果销售物料有冻结、订单，那么物料类型不能变
        if($shop_id != '_ALL_'){
            $shopFreezeObj = app::get('material')->model('sales_material_shop_freeze');
            $storeInfo = $shopFreezeObj->getList('shop_freeze',array('sm_id'=>$sm_id,'shop_id'=>$shop_id));
            if($storeInfo[0]['shop_freeze'] > 0){
                $is_type_readonly = true;
            }
        }

        $orderObjObj = app::get('ome')->model('order_objects');
        $orderInfo = $orderObjObj->getList('obj_id,goods_id',array('goods_id'=>$sm_id));
        if($orderInfo){
            $has_object = true;
            $is_type_readonly = true;
            $is_shop_readonly = true;
        }

        if($is_type_readonly){
            $readonly['type'] = true;
        }

        if($is_shop_readonly){
            $readonly['shop'] = true;
        }

        //已绑定有订单的不能变
        if($is_bind == 1 && $has_object == true){
            $orderItemObj = app::get('ome')->model('order_items');
            $orderItemInfo = $orderItemObj->db->select("select item_id from sdb_ome_order_items as oi left join sdb_ome_order_objects as oo on oi.obj_id = oo.obj_id where goods_id = ".$sm_id." LIMIT 0,1");
            if($orderItemInfo){
                $is_bind_item_readonly = true;
            }
        }

        if($is_bind_item_readonly){
            $readonly['bind_item'] = true;
        }

        return $readonly;
    }
}