<?php
/**
 * 销售物料Lib类
 * @author xiayuanjun@shopex.cn
 * @version 1.0
 */
class material_sales_material{

    function __construct(){
        $this->_salesMaterialObj = app::get('material')->model('sales_material');
        $this->_salesBasicMaterialObj = app::get('material')->model('sales_basic_material');
        $this->_basicMaterialObj = app::get('material')->model('basic_material');
        $this->_basicMaterialExtObj = app::get('material')->model('basic_material_ext');
        $this->_basicMaterialStore  = app::get('ome')->model('branch_product');
    }

    /**
     *
     * 根据来源店铺及销售物料货号获取销售物料信息
     * @param String $shop_id
     * @param String $bn
     * @return Boolean
     */
    public function getSalesMByBn($shop_id, $bn){
        $salesMaterialInfo = $this->_salesMaterialObj->getList('sm_id,sales_material_name,sales_material_bn,shop_id,is_bind,sales_material_type',array('sales_material_bn'=>$bn,'shop_id'=>array($shop_id,'_ALL_')), 0, 1);
        if($salesMaterialInfo){
            return $salesMaterialInfo[0];
        }
        return false;
    }

    /**
     *
     * 根据来源店铺及销售物料ID获取销售物料信息
     * @param String $shop_id
     * @param Int $sm_id
     * @return Boolean
     */
    public function getSalesMById($shop_id, $sm_id){
        $salesMaterialInfo = $this->_salesMaterialObj->getList('sm_id,sales_material_name,sales_material_bn,shop_id,is_bind,sales_material_type',array('sm_id'=>$sm_id,'shop_id'=>array($shop_id,'_ALL_')), 0, 1);
        if($salesMaterialInfo){
            return $salesMaterialInfo[0];
        }
        return false;
    }

    /**
     *
     * 根据来源店铺及货号获取销售物料信息
     * @param String $shop_id
     * @param String $bn
     * @return Boolean
     */
    public function getBasicMBySalesMId($sm_id){
        $salesBasicMInfos = $this->_salesBasicMaterialObj->getList('bm_id,sm_id,number,rate',array('sm_id'=>$sm_id), 0, -1);
        if($salesBasicMInfos){
            foreach($salesBasicMInfos as $k => $salesBasicMInfo){
                $bmIds[] = $salesBasicMInfo['bm_id'];
                $bmAndSmRates[$salesBasicMInfo['bm_id']] = $salesBasicMInfo;
            }

            $basicMaterialExtInfos = $this->_basicMaterialExtObj->getList('bm_id,cost,weight,retail_price,unit',array('bm_id'=>$bmIds), 0, -1);
            if($basicMaterialExtInfos){
                foreach($basicMaterialExtInfos as $key => $basicMaterialExtInfo){
                    $bmExts[$basicMaterialExtInfo['bm_id']] = $basicMaterialExtInfo;
                }
            }

            $basicMaterialInfos = $this->_basicMaterialObj->getList('bm_id,material_name,material_bn',array('bm_id'=>$bmIds), 0, -1);
            if($basicMaterialInfos){
                foreach($basicMaterialInfos as $key => $basicMaterialInfo){
                    $bmList[] = array_merge($basicMaterialInfo, $bmAndSmRates[$basicMaterialInfo['bm_id']], $bmExts[$basicMaterialInfo['bm_id']]);
                }
                return $bmList;
            }
        }
        return false;
    }

    /**
     *
     * 根据促销总价格计算每个物料的贡献金额值
     * @param String $shop_id
     * @param String $bn
     * @return Boolean
     */
    public function calProSaleMPriceByRate($sale_price, &$bm_bns){
        if($sale_price <=0){
            foreach($bm_bns as $k =>$bm_bn){
                $bm_bns[$k]['rate_price'] = 0.00;
            }
            return true;
        }

        $less_price = $sale_price;
        $count_sku = count($bm_bns);
        $i = 1;
        foreach($bm_bns as $k =>$bm_bn){
            if($i == $count_sku){
                $bm_bns[$k]['rate_price'] = $less_price;
            }else{
                $tmp_rate = $bm_bn['rate']/100;
                $bm_bns[$k]['rate_price'] = bcmul($sale_price, $tmp_rate, 2);
                $less_price -= $bm_bns[$k]['rate_price'];
            }
            $i++;
        }

        return true;
    }

    /**
     * 可删除，没有地方调用!!!
     * 
     * @param $sm_id
     * @param $salse_num
     * @return bool
     */
    public function validSalesBasicMaterial($sm_id, $salse_num){
        $relations = $this->_salesBasicMaterialObj->getList('sm_id, bm_id, number', array('sm_id'=>$sm_id), 0, -1);
        foreach($relations as $k => $v){
            $all_nums = $salse_num * $v['number'];
            $stores = $this->_basicMaterialStore->dump(array('product_id' => $v['bm_id']));
            
            //团购导入时,可用库存 = 总库存 - 冻结库存
            $store_num    = $stores['store'] - $stores['store_freeze'];
            if($store_num - $all_nums < 0){
                return false;
            }
        }
        return true;

    }

    /**
     *
     * 根据来源店铺及货号获取促销[捆绑]销售物料信息
     * @param String $shop_id
     * @param String $bn
     * @return Boolean
     */
    public function getBasicMBySalesMIds($sm_id)
    {
        $salesBasicMInfos = $this->_salesBasicMaterialObj->getList('bm_id,sm_id,number,rate',array('sm_id'=>$sm_id), 0, -1);
        if($salesBasicMInfos)
        {
            foreach($salesBasicMInfos as $k => $salesBasicMInfo)
            {
                $bmIds[] = $salesBasicMInfo['bm_id'];
                $bmAndSmRates[$salesBasicMInfo['sm_id']][$salesBasicMInfo['bm_id']] = $salesBasicMInfo;
            }
            
            $bmExts    = array();
            $basicMaterialExtInfos = $this->_basicMaterialExtObj->getList('bm_id,cost,weight,retail_price,unit',array('bm_id'=>$bmIds), 0, -1);
            if($basicMaterialExtInfos)
            {
                foreach($basicMaterialExtInfos as $key => $basicMaterialExtInfo)
                {
                    $bmExts[$basicMaterialExtInfo['bm_id']] = $basicMaterialExtInfo;
                }
            }
            
            $basic_material_infos    = array();
            $basicMaterialInfos = $this->_basicMaterialObj->getList('bm_id,material_name,material_bn',array('bm_id'=>$bmIds), 0, -1);
            if($basicMaterialInfos)
            {
                foreach($basicMaterialInfos as $key => $basicMaterialInfo)
                {
                    $basic_material_infos[$basicMaterialInfo['bm_id']]    = $basicMaterialInfo;
                }
            }
            
            #合并信息
            $bmList    = array();
            foreach ($bmAndSmRates as $sm_key => $sales_basic_material_list)
            {
                foreach ($sales_basic_material_list as $bm_key => $bm_item)
                {
                    $bmList[$sm_key][]    = array_merge($basic_material_infos[$bm_key], $bm_item, $bmExts[$bm_key]);
                }
            }
            
            return $bmList;
        }
        return false;
    }

    /**
     *
     * 根据分销总价格计算每个物料的贡献金额值
     * 
     * 
     * @return array
     */
    public function calProbuyerMPriceByRate($shop_id,$buyer_payment,$obj_bn){
        
        //检查货品是否存在销售物料中
        $salesMInfo = $this->getSalesMByBn($shop_id,$obj_bn);

        $bm_bns = array();
        if($salesMInfo){
            $basicMInfos = $this->getBasicMBySalesMId($salesMInfo['sm_id']);
            if($basicMInfos) {
                if($buyer_payment <=0){
                    foreach($basicMInfos as $bm){
                        $bm_bns[$bm['material_bn']]['rate_price'] = 0.00;
                    }

                }else{
                    $less_price = $buyer_payment;
                    $count_sku = count($basicMInfos);
                    $i = 1;
                    foreach($basicMInfos as $bm){
                        if($i == $count_sku){
                            $bm_bns[$bm['material_bn']]['rate_price'] = $less_price;
                        }else{
                            $tmp_rate = $bm['rate']/100;
                            $bm_bns[$bm['material_bn']]['rate_price'] = bcmul($buyer_payment, $tmp_rate, 2);
                            $less_price -= $bm_bns[$bm['material_bn']]['rate_price'];
                        }
                        $i++;
                    }
                }


            }
        }
        return $bm_bns;


    }

    /**
     *
     * 根据销售物料ID数组获取对应的基础物料ID数组
     * @param array $sm_ids
     * @return Array
     */
    public function getBmIdsBySmIds($sm_ids){
        $sm_and_bms = array();
        $salesBasicMInfos = $this->_salesBasicMaterialObj->getList('bm_id,sm_id',array('sm_id'=>$sm_ids), 0, -1);
        if($salesBasicMInfos){
            foreach($salesBasicMInfos as $salesBasicMInfo){
                if(isset($sm_and_bms[$salesBasicMInfo['sm_id']])){
                    $sm_and_bms[$salesBasicMInfo['sm_id']] = array_merge((array)$sm_and_bms[$salesBasicMInfo['sm_id']], (array)$salesBasicMInfo['bm_id']);
                }else{
                    $sm_and_bms[$salesBasicMInfo['sm_id']] = $salesBasicMInfo['bm_id'];
                }
            }
            return $sm_and_bms;
        }
    }
    
    //根据销售物料sm_id获取信息福袋信息
    public function get_luckybag_by_sm_id($sm_id){
        $mdl_ma_lu_ru = app::get('material')->model('luckybag_rules');
        $mdl_ma_ba_ma = app::get('material')->model('basic_material');
        $mdl_ma_ba_ma_ext = app::get('material')->model('basic_material_ext');
        $rs = $mdl_ma_lu_ru->getList("*",array("sm_id"=>$sm_id));
        if(empty($rs)){
            return array();
        }
        $return_arr = array();
        $load_part_number = 1;
        foreach($rs as $var_rs){
            $temp_bm_ids = explode(",",$var_rs["bm_ids"]);
            $rs_bm_info = $mdl_ma_ba_ma->getList("bm_id,material_bn,material_name",array("bm_id"=>$temp_bm_ids));
            $rs_bm_info_ext = $mdl_ma_ba_ma_ext->getList("bm_id,cost",array("bm_id"=>$temp_bm_ids));
            $rl_bm_id_ext = array();
            foreach($rs_bm_info_ext as $var_rs_bie){
                $rl_bm_id_ext[$var_rs_bie["bm_id"]]["cost"] = $var_rs_bie["cost"];
            }
            $bm_list = array();
            foreach($rs_bm_info as $var_rbi){
                $bm_list[] = array(
                    "bm_id" => $var_rbi["bm_id"],
                    "material_bn" => $var_rbi["material_bn"],
                    "material_name" => $var_rbi["material_name"],
                    "cost" => $rl_bm_id_ext[$var_rbi["bm_id"]]["cost"],
                );
            }
            $return_arr[] = array(
                "lbr_id" =>  $var_rs["lbr_id"],
                "lbr_name" =>  $var_rs["lbr_name"],
                "bm_ids" =>  $var_rs["bm_ids"],
                "sku_num" =>  $var_rs["sku_num"],
                "send_num" =>  $var_rs["send_num"],
                "price" =>  $var_rs["price"],
                "bm_list" =>  $bm_list,
                "load_part_number" => $load_part_number,
                "current_retail_price" => $var_rs["sku_num"]*$var_rs["send_num"]*$var_rs["price"],
            );
            $load_part_number++;
        }
        return $return_arr;
    }
    
    //获取订单福袋明细数据
    public function get_order_luckybag_bminfo($sm_id){
        $mdl_ma_lu_ru = app::get('material')->model('luckybag_rules');
        $mdl_ma_ba_ma_st = app::get('material')->model('basic_material_stock');
        $lib_ma_ba_ma_st_fr  = kernel::single('material_basic_material_stock_freeze');
        $rs_luckybag_groups = $mdl_ma_lu_ru->getList("*",array("sm_id"=>$sm_id));
        $return_bm_id_info = array();
        foreach($rs_luckybag_groups as $var_rlgs){
            $current_bm_ids = explode(",", $var_rlgs["bm_ids"]);
            if(count($current_bm_ids) == $var_rlgs["sku_num"]){ //当前bm总数量和sku数量一致 不用随机 直接全部拿
                $this->get_order_luckybag_single_bminfo($current_bm_ids,$var_rlgs["lbr_id"],$var_rlgs["send_num"],$var_rlgs["price"],$return_bm_id_info);
            }else{
                //获取bm可用库存
                $current_stock_info = $mdl_ma_ba_ma_st->getList('bm_id,store,store_freeze',array('bm_id'=>$current_bm_ids));
                $current_real_store = array();
                foreach($current_stock_info as $var_csi){
                    $var_csi['store_freeze'] = $lib_ma_ba_ma_st_fr->getMaterialStockFreeze($var_csi['bm_id']);
                    $tmp_abled_store = ($var_csi['store']-$var_csi['store_freeze']) > 0 ? ($var_csi['store']-$var_csi['store_freeze']) : 0;
                    $current_real_store[$var_csi["bm_id"]] = $tmp_abled_store > 0 ? floor($tmp_abled_store/$var_rlgs["send_num"]) : 0;
                }
                //随机数组 后获取库存足和不足的两个数组
                $count_arr = count($current_real_store);
                $has_stock_bm_ids = array();
                $hasnot_stock_bm_ids = array();
                for($temp_number=1;$temp_number<=$count_arr;$temp_number++){
                    $rand_bm_id = array_rand($current_real_store);
                    if($current_real_store[$rand_bm_id] > 0){
                        $has_stock_bm_ids[] = $rand_bm_id;
                    }else{
                        $hasnot_stock_bm_ids[] = $rand_bm_id;
                    }
                    unset($current_real_store[$rand_bm_id]);
                }
                $count_has_stock_bm_ids = count($has_stock_bm_ids);
                if($count_has_stock_bm_ids == $var_rlgs["sku_num"]){ //sku数量和有库存的bm数量一样时 直接拿有库存的基础物料即可
                    $this->get_order_luckybag_single_bminfo($has_stock_bm_ids,$var_rlgs["lbr_id"],$var_rlgs["send_num"],$var_rlgs["price"],$return_bm_id_info);
                }elseif($count_has_stock_bm_ids > $var_rlgs["sku_num"]){ //有库存的bm数量大于sku数量时 直接拿相应sku数量的有库存的基础物料即可
                    $chunk_arr = array_chunk($has_stock_bm_ids,$var_rlgs["sku_num"]);
                    $result_bm_ids = array();
                    foreach($chunk_arr[0] as $var_ca){
                        $result_bm_ids[] = $var_ca;
                    }
                    $this->get_order_luckybag_single_bminfo($result_bm_ids,$var_rlgs["lbr_id"],$var_rlgs["send_num"],$var_rlgs["price"],$return_bm_id_info);
                }elseif($count_has_stock_bm_ids < $var_rlgs["sku_num"]){
                    //少的sku数量用库存不足的补上
                    $less_number = $var_rlgs["sku_num"] - $count_has_stock_bm_ids;
                    $chunk_arr = array_chunk($hasnot_stock_bm_ids,$less_number);
                    //把库存不足的补上后 获取数据
                    foreach($chunk_arr[0] as $var_ca){
                        $has_stock_bm_ids[] = $var_ca;
                    }
                    $this->get_order_luckybag_single_bminfo($has_stock_bm_ids,$var_rlgs["lbr_id"],$var_rlgs["send_num"],$var_rlgs["price"],$return_bm_id_info);
                }
            }
        }
        return $return_bm_id_info;
    }
    
    //组装福袋基础物料信息
    function get_order_luckybag_single_bminfo($bm_ids,$lbr_id,$send_num,$price,&$return_bm_id_info){
        $mdl_ma_ba_ma = app::get('material')->model('basic_material');
        $mdl_ma_ba_ma_ext = app::get('material')->model('basic_material_ext');
        $rs_ba = $mdl_ma_ba_ma->getList("bm_id,material_name,material_bn,type",array("bm_id"=>$bm_ids));
        $rs_ba_ext = $mdl_ma_ba_ma_ext->getList("bm_id,cost,weight",array("bm_id"=>$bm_ids));
        //保持bm_ids顺序获取数据
        $rl_bm_id_info_ext = array();
        foreach($rs_ba_ext as $var_be){
            $rl_bm_id_info_ext[$var_be["bm_id"]] = array(
                "weight" => $var_be["weight"],
                "cost" => $var_be["cost"],
            );
        }
        $rl_bm_id_info = array();
        foreach($rs_ba as $var_rb){
            $rl_bm_id_info[$var_rb["bm_id"]] = $var_rb;
        }
        foreach($bm_ids as $var_bi){
            $return_bm_id_info[] = array(
                "lbr_id" => $lbr_id,
                "bm_id" => $var_bi,
                "material_name" => $rl_bm_id_info[$var_bi]["material_name"],
                "material_bn" => $rl_bm_id_info[$var_bi]["material_bn"],
                "number" => $send_num, //参考原有代码用number
                "price" => $price,
                "weight" => $rl_bm_id_info_ext[$var_bi]["weight"],
                "cost" => $rl_bm_id_info_ext[$var_bi]["cost"],
                "type" => $rl_bm_id_info[$var_bi]["type"] //售后getProducts方法用了先带上
            );
        }
    }

}